library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity sound_controller is
port (

    clk        : in  std_logic;
    reset_n    : in  std_logic;
    read       : in  std_logic;
    write      : in  std_logic;
    chipselect : in  std_logic;
    address    : in  std_logic_vector(3 downto 0);
    readdata   : out std_logic_vector(15 downto 0);
    writedata  : in  std_logic_vector(15 downto 0);
    audioIrq   : out std_logic;
	 
	 audio_data 	: in  std_logic_vector(15 downto 0);
	 audio_address : out  std_logic_vector(16 downto 0);
	 
	AUD_ADCLRCK  : out  std_logic;   --    Audio CODEC ADC LR Clock
	AUD_ADCDAT   : in   std_logic;   --    Audio CODEC ADC Data
	AUD_DACLRCK  : out  std_logic;   --    Audio CODEC DAC LR Clock
	AUD_DACDAT   : out  std_logic;   --    Audio CODEC DAC Data
	AUD_BCLK     : inout std_logic --    Audio CODEC Bit-Stream Clock
  );
  
  end sound_controller;
  
  architecture rtl of sound_controller is
  
  signal clk_25:std_logic;
  signal counter: unsigned (7 downto 0);
  signal count: unsigned (7 downto 0);
  type 	ram_type is array (0 to 255) of unsigned (17 downto 0);
  signal buf1 : ram_type;
  signal buf2 : ram_type;
  signal data: unsigned (15 downto 0);
  signal mode:std_logic := '0'; 
  signal ram_address : unsigned(7 downto 0) := "00000000";
  signal audio_offset : unsigned(15 downto 0);
  signal base_address : unsigned(15 downto 0);
  signal write_en: std_logic_vector(31 downto 0) := X"00000000"; 
  signal sound_next: std_logic_vector(31 downto 0) := X"00000000"; 
  signal sound_en_num: unsigned(4 downto 0) := "00000";  
  signal current_sound: integer range 0 to 30;
  signal current_rom: integer range 0 to 8; 
  signal current_length: integer range 0 to 15; 
  signal current_play: integer range 0 to 30; 
  
  type aud_address_type is array(integer range 0 to 30) of unsigned(15 downto 0);
  signal aud_address : aud_address_type;
  signal sound_length : aud_address_type;
  signal sound_base : aud_address_type;
  
  type aud_address_base_type is array(integer range 0 to 7) of unsigned(15 downto 0);
  constant aud_address_base : aud_address_base_type := ( 
  X"0000", X"0600", X"0A00", X"2600",
  X"3A00", X"4D00", X"0000", X"0000"
  );
  
  type aud_address_length_type is array(integer range 0 to 15) of unsigned(15 downto 0);
  signal aud_address_length : aud_address_length_type := ( 
  X"05FF", X"03FF", X"13FF", X"13FF",
  X"02FF", X"01FF", X"07FF", X"09FF",
  X"02FF", X"01FF", X"05FF", X"07FF",
  X"12FF", X"097F", X"097F", X"09FF"
  );
  
  type state is (s0,s1,s2,s3);--,s4,s5,s6,s7,s8,s9,s10,s11,s12,s13,s14,s15,s16,s17,s18,s19,s20,s21,s22);
  signal current_state : state;

   component de2_wm8731_audio is port(
		    clk 				: in std_logic;       --  Audio CODEC Chip Clock AUD_XCK (18.43 MHz)
    		 reset_n 		: in std_logic;
		    data      		: in unsigned(15 downto 0);
			 counter			: buffer unsigned (7 downto 0);
  
		    -- Audio interface signals
    		 AUD_ADCLRCK  	: out  std_logic;   --    Audio CODEC ADC LR Clock
		    AUD_ADCDAT   	: in   std_logic;   --    Audio CODEC ADC Data
		    AUD_DACLRCK  	: out  std_logic;   --    Audio CODEC DAC LR Clock
		    AUD_DACDAT   	: out  std_logic;   --    Audio CODEC DAC Data
		    AUD_BCLK     	: inout std_logic	 --    Audio CODEC Bit-Stream Clock
	);
	end component de2_wm8731_audio;

begin

	audio_address(15 downto 0) <= std_logic_vector(audio_offset + base_address);
	
	current_sound <= to_integer(unsigned(writedata(5 downto 1)));
	current_rom <= to_integer(unsigned(writedata(9 downto 6)));
	current_length <= to_integer(unsigned(writedata(13 downto 10)));
	
	process (current_state) 
	begin
		case current_state is
		when s1 =>
			audio_offset <= aud_address(current_play);
			base_address <= sound_base(current_play);
		when others =>
		end case;
	end process;
	
	process (clk)
	begin 
		if rising_edge (clk) then
			clk_25 <= not clk_25;
		end if;
	end process;

	process (clk)
	begin
		if rising_edge(clk) then
			if reset_n = '0' then
				audioIrq <= '0';
			else
			
				-- generate irq
				if count = "11111111" and counter = "00000000" then
					audioIrq <= '1';
					mode <= not mode;
					ram_address <= (others => '0');
					current_state <= s0;
					current_play <= 0;
					
				-- clear irq
				elsif write = '1' and chipselect = '1' then
					audioIrq <= '0';
					if (sound_next(current_sound) = '0' and writedata(0) = '1') or (sound_next(current_sound) = '1' and writedata(0) = '0') then
						write_en(current_sound) <= '1';
						sound_base(current_sound) <= aud_address_base(current_rom);
						sound_length(current_sound) <= aud_address_length(current_length);
					end if;
					sound_next(current_sound) <= writedata(0);
				end if;
				count <= counter;
				
				case current_state is
					when s0 =>	
						if ram_address = "11111111" then
							current_state <= s1;
						end if;	
						ram_address <= ram_address + 1;
						sound_en_num <= (others => '0');
						
					when s1 =>						
						if aud_address(current_play) >= sound_length(current_play) then
							aud_address(current_play) <= (others => '0');
							write_en(current_play) <= '0';
						end if;
						
						if ram_address = "11111111" then
							if write_en(current_play) = '1' and current_play > 9 then
								sound_en_num <= sound_en_num + 1;
							end if;
							current_play <= current_play + 1;
						else
							if write_en(current_play) = '1' then
								aud_address(current_play) <= aud_address(current_play) + 1;
							end if;
						end if;				
				
						if current_play = 26 then
							current_state <= s2;
						end if;
						ram_address <= ram_address + 1;
						
					when s2 => 
						if ram_address = "11111111" then
							current_state <= s3;
						end if;	
						ram_address <= ram_address + 1;
						
					when s3 => 	
				end case;		
			end if;
		end if;
	end process;

	
	data <= buf1(to_integer (counter))(15 downto 0) when mode = '0' else buf2(to_integer (counter))(15 downto 0);
	process (clk)
	begin
		if rising_edge(clk) then
			case current_state is
				when s0 =>	
					if mode = '0' then
						buf2 (to_integer (unsigned(ram_address))) <= (others => '0');
					elsif mode = '1' then
						buf1 (to_integer (unsigned(ram_address))) <= (others => '0');
					end if;						
				when s1 =>	
					if write_en(current_play) = '1' and mode = '0' then
						buf2 (to_integer (unsigned(ram_address))) <= buf2 (to_integer (unsigned(ram_address))) 
						+ unsigned(audio_data(15)&audio_data(15)&audio_data(15 downto 0));
					elsif write_en(current_play) = '1' and mode = '1' then
						buf1 (to_integer (unsigned(ram_address))) <= buf1 (to_integer (unsigned(ram_address))) 
						+ unsigned(audio_data(15)&audio_data(15)&audio_data(15 downto 0));
					end if;	
				when s2 =>
					case sound_en_num is
--						when "00010" =>
--							if mode = '0' then
--								buf2 (to_integer (unsigned(ram_address))) <= "0" & buf2 (to_integer (unsigned(ram_address)))(17 downto 1);
--							elsif mode = '1' then
--								buf1 (to_integer (unsigned(ram_address))) <= "0" & buf1 (to_integer (unsigned(ram_address)))(17 downto 1);
--							end if;
--						when "00011" =>
--							if mode = '0' then
--								buf2 (to_integer (unsigned(ram_address))) <= "00" & buf2 (to_integer (unsigned(ram_address)))(17 downto 2);
--							elsif mode = '1' then
--								buf1 (to_integer (unsigned(ram_address))) <= "00" & buf1 (to_integer (unsigned(ram_address)))(17 downto 2);
--							end if;
--						when "00100" =>
--							if mode = '0' then
--								buf2 (to_integer (unsigned(ram_address))) <= "00" & buf2 (to_integer (unsigned(ram_address)))(17 downto 2);
--							elsif mode = '1' then
--								buf1 (to_integer (unsigned(ram_address))) <= "00" & buf1 (to_integer (unsigned(ram_address)))(17 downto 2);
--							end if;
						when others =>
					end case;
				when s3 =>
			end case;
		end if;
	end process;

	V1: de2_wm8731_audio port map (
    clk 				=> clk_25,
    reset_n   		=> '1',        
    data 			=>  data,
	 counter 		=> counter,
  
    -- Audio interface signals
    AUD_ADCLRCK  	=> AUD_ADCLRCK,
    AUD_ADCDAT   	=> AUD_ADCDAT,
    AUD_DACLRCK  	=> AUD_DACLRCK,
    AUD_DACDAT   	=> AUD_DACDAT,
    AUD_BCLK     	=> AUD_BCLK
  );
  
  end architecture;
