--
-- DE2 top-level module that includes the simple VGA raster generator
--
-- Stephen A. Edwards, Columbia University, sedwards@cs.columbia.edu
--
-- From an original by Terasic Technology, Inc.
-- (DE2_TOP.v, part of the DE2 system board CD supplied by Altera)
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity top_level is

  port (
    -- Clocks
    
    CLOCK_27,                                      -- 27 MHz
    CLOCK_50,                                      -- 50 MHz
    EXT_CLOCK : in std_logic;                      -- External Clock

    -- Buttons and switches
    
    KEY : in std_logic_vector(3 downto 0);         -- Push buttons
    SW : in std_logic_vector(17 downto 0);         -- DPDT switches

    -- LED displays

    HEX0, HEX1, HEX2, HEX3, HEX4, HEX5, HEX6, HEX7 -- 7-segment displays
       : out std_logic_vector(6 downto 0);
    LEDG : out std_logic_vector(8 downto 0);       -- Green LEDs
    LEDR : out std_logic_vector(17 downto 0);      -- Red LEDs

    -- RS-232 interface

    UART_TXD : out std_logic;                      -- UART transmitter   
    UART_RXD : in std_logic;                       -- UART receiver

    -- IRDA interface

--    IRDA_TXD : out std_logic;                      -- IRDA Transmitter
    IRDA_RXD : in std_logic;                       -- IRDA Receiver

    -- SDRAM
   
    DRAM_DQ : inout std_logic_vector(15 downto 0); -- Data Bus
    DRAM_ADDR : out std_logic_vector(11 downto 0); -- Address Bus    
    DRAM_LDQM,                                     -- Low-byte Data Mask 
    DRAM_UDQM,                                     -- High-byte Data Mask
    DRAM_WE_N,                                     -- Write Enable
    DRAM_CAS_N,                                    -- Column Address Strobe
    DRAM_RAS_N,                                    -- Row Address Strobe
    DRAM_CS_N,                                     -- Chip Select
    DRAM_BA_0,                                     -- Bank Address 0
    DRAM_BA_1,                                     -- Bank Address 0
    DRAM_CLK,                                      -- Clock
    DRAM_CKE : out std_logic;                      -- Clock Enable

    -- FLASH
    
    FL_DQ : inout std_logic_vector(7 downto 0);      -- Data bus
    FL_ADDR : out std_logic_vector(21 downto 0);  	  -- Address bus
    FL_WE_N,                                         -- Write Enable
    FL_RST_N,                                        -- Reset
    FL_OE_N,                                         -- Output Enable
    FL_CE_N : out std_logic;                         -- Chip Enable

    -- SRAM
    
    SRAM_DQ : inout std_logic_vector(15 downto 0); -- Data bus 16 Bits
    SRAM_ADDR : out std_logic_vector(17 downto 0); -- Address bus 18 Bits
    SRAM_UB_N,                                     -- High-byte Data Mask 
    SRAM_LB_N,                                     -- Low-byte Data Mask 
    SRAM_WE_N,                                     -- Write Enable
    SRAM_CE_N,                                     -- Chip Enable
    SRAM_OE_N : out std_logic;                     -- Output Enable

    -- USB controller
    
    OTG_DATA : inout std_logic_vector(15 downto 0); -- Data bus
    OTG_ADDR : out std_logic_vector(1 downto 0);    -- Address
    OTG_CS_N,                                       -- Chip Select
    OTG_RD_N,                                       -- Write
    OTG_WR_N,                                       -- Read
    OTG_RST_N,                                      -- Reset
    OTG_FSPEED,                     -- USB Full Speed, 0 = Enable, Z = Disable
    OTG_LSPEED : out std_logic;     -- USB Low Speed, 0 = Enable, Z = Disable
    OTG_INT0,                                       -- Interrupt 0
    OTG_INT1,                                       -- Interrupt 1
    OTG_DREQ0,                                      -- DMA Request 0
    OTG_DREQ1 : in std_logic;                       -- DMA Request 1   
    OTG_DACK0_N,                                    -- DMA Acknowledge 0
    OTG_DACK1_N : out std_logic;                    -- DMA Acknowledge 1

    -- 16 X 2 LCD Module
    
    LCD_ON,                     -- Power ON/OFF
    LCD_BLON,                   -- Back Light ON/OFF
    LCD_RW,                     -- Read/Write Select, 0 = Write, 1 = Read
    LCD_EN,                     -- Enable
    LCD_RS : out std_logic;     -- Command/Data Select, 0 = Command, 1 = Data
    LCD_DATA : inout std_logic_vector(7 downto 0); -- Data bus 8 bits

    -- SD card interface
    
    SD_DAT,                     -- SD Card Data
    SD_DAT3,                    -- SD Card Data 3
    SD_CMD : inout std_logic;   -- SD Card Command Signal
    SD_CLK : out std_logic;     -- SD Card Clock

    -- USB JTAG link
    
    TDI,                        -- CPLD -> FPGA (data in)
    TCK,                        -- CPLD -> FPGA (clk)
    TCS : in std_logic;         -- CPLD -> FPGA (CS)
    TDO : out std_logic;        -- FPGA -> CPLD (data out)

    -- I2C bus
    
    I2C_SDAT : inout std_logic; -- I2C Data
    I2C_SCLK : out std_logic;   -- I2C Clock

    -- PS/2 port

    PS2_DAT : inout std_logic;     -- Data
    PS2_CLK : inout std_logic;     -- Clock

    -- VGA output
    
    VGA_CLK,                                            -- Clock
    VGA_HS,                                             -- H_SYNC
    VGA_VS,                                             -- V_SYNC
    VGA_BLANK,                                          -- BLANK
    VGA_SYNC : out std_logic;                           -- SYNC
    VGA_R,                                              -- Red[9:0]
    VGA_G,                                              -- Green[9:0]
    VGA_B : out std_logic_vector(9 downto 0);                   -- Blue[9:0]

    --  Ethernet Interface
    
    ENET_DATA : inout std_logic_vector(15 downto 0);    -- DATA bus 16Bits
    ENET_CMD,           -- Command/Data Select, 0 = Command, 1 = Data
    ENET_CS_N,                                          -- Chip Select
    ENET_WR_N,                                          -- Write
    ENET_RD_N,                                          -- Read
    ENET_RST_N,                                         -- Reset
    ENET_CLK : out std_logic;                           -- Clock 25 MHz
    ENET_INT : in std_logic;                            -- Interrupt
    
    -- Audio CODEC
    
    AUD_ADCLRCK : inout std_logic;                      -- ADC LR Clock
    AUD_ADCDAT : in std_logic;                          -- ADC Data
    AUD_DACLRCK : inout std_logic;                      -- DAC LR Clock
    AUD_DACDAT : out std_logic;                         -- DAC Data
    AUD_BCLK : inout std_logic;                         -- Bit-Stream Clock
    AUD_XCK : out std_logic;                            -- Chip Clock
    
    -- Video Decoder
    
    TD_DATA : in std_logic_vector(7 downto 0);  -- Data bus 8 bits
    TD_HS,                                      -- H_SYNC
    TD_VS : in std_logic;                       -- V_SYNC
    TD_RESET : out std_logic;                   -- Reset
    
    -- General-purpose I/O
    
    GPIO_0,                                      -- GPIO Connection 0
    GPIO_1 : inout std_logic_vector(35 downto 0) -- GPIO Connection 1   
    );
  
end top_level;

architecture datapath of top_level is

  component de2_i2c_av_config is
  port (
    iCLK : in std_logic;
    iRST_N : in std_logic;
    I2C_SCLK : out std_logic;
    I2C_SDAT : inout std_logic
  );
  end component;  
  
  signal counter : unsigned(4 downto 0);
  signal reset_n : std_logic;

  signal BA : STD_LOGIC_VECTOR(1 DOWNTO 0);
  signal DQM : STD_LOGIC_VECTOR(1 DOWNTO 0);
  signal pll_c1: STD_LOGIC;
  signal ram_addr  : std_logic_vector(17 downto 0);
  signal ram_data  : std_logic_vector(15 downto 0);
  signal vga_addr  : std_logic_vector(17 downto 0);
  signal vga_data  : std_logic_vector(15 downto 0);
  signal read_ready: std_logic_vector(15 downto 0);
  signal SRAM_address_io: std_logic_vector(17 downto 0);
  signal SRAM_byteenable_io: std_logic_vector(1 downto 0);
  signal SRAM_chipselect_io: std_logic; 
  signal SRAM_read_io: std_logic;
  signal SRAM_write_io: std_logic;
  signal SRAM_writedata_io: std_logic_vector(15 downto 0);
  signal clk25 : std_logic := '0';
  signal sprite_pixel : unsigned(7 downto 0); 
  signal sprite_y_blank :  std_logic;
  signal sprite_x : unsigned(9 downto 0);
  signal sprite_y : unsigned(9 downto 0);
  signal sprite_irq : std_logic;
  signal sprite_write :  std_logic;
  signal sprite_writedata : std_logic_vector(31 downto 0);
  signal sprite_address : std_logic_vector(15 downto 0);
  signal mouse_data : std_logic_vector(31 downto 0);
  signal audio_data : std_logic_vector(15 downto 0);
  signal audio_address : std_logic_vector(16 downto 0);
  signal audio_clock : unsigned(1 downto 0) := "00";
  signal x_cnt   : unsigned(11 downto 0);
 
  begin

  process (CLOCK_50)
  begin
    if rising_edge(CLOCK_50) then
	   clk25 <= not clk25;
      if counter = "1111" then
        reset_n <= '1';
      else
        reset_n <= '0';
        counter <= counter + 1;
      end if;
    end if;
  end process;

  AUD_XCK <= clk25;

  i2c : de2_i2c_av_config port map (
    iCLK     => CLOCK_50,
    iRST_n   => '1',
    I2C_SCLK => I2C_SCLK,
    I2C_SDAT => I2C_SDAT
  );

  nios : entity work.nios_system port map (
    clk_0                        => pll_c1,
    reset_n                      => '1',
	 
	 AUD_ADCDAT_to_the_sound_ctrl        => AUD_ADCDAT,
	 AUD_ADCLRCK_from_the_sound_ctrl     => AUD_ADCLRCK,
	 AUD_BCLK_to_and_from_the_sound_ctrl => AUD_BCLK,
	 AUD_DACDAT_from_the_sound_ctrl      => AUD_DACDAT,
	 AUD_DACLRCK_from_the_sound_ctrl     => AUD_DACLRCK,
	 audio_address_from_the_sound_ctrl 	 => audio_address,
	 audio_data_to_the_sound_ctrl 		 => audio_data,
	 
    SRAM_address_from_the_sram_ctrl    => SRAM_address_io,
    SRAM_byteenable_from_the_sram_ctrl => SRAM_byteenable_io,
    SRAM_chipselect_from_the_sram_ctrl => SRAM_chipselect_io,
    SRAM_read_from_the_sram_ctrl       => SRAM_read_io,
    SRAM_write_from_the_sram_ctrl      => SRAM_write_io,
    SRAM_writedata_from_the_sram_ctrl  => SRAM_writedata_io,
	 
	 rdaddress_to_the_ram_ctrl 			=> audio_address(14 downto 0),
	 q_from_the_ram_ctrl 					=> audio_data,

	 zs_addr_from_the_sdram_ctrl        => DRAM_ADDR,
	 zs_ba_from_the_sdram_ctrl          => BA,
	 zs_cas_n_from_the_sdram_ctrl       => DRAM_CAS_N,
	 zs_cke_from_the_sdram_ctrl         => DRAM_CKE,
	 zs_cs_n_from_the_sdram_ctrl        => DRAM_CS_N,
	 zs_dq_to_and_from_the_sdram_ctrl   => DRAM_DQ,
	 zs_dqm_from_the_sdram_ctrl         => DQM,
	 zs_ras_n_from_the_sdram_ctrl       => DRAM_RAS_N,
	 zs_we_n_from_the_sdram_ctrl        => DRAM_WE_N,

    address_out_from_the_sprite_ctrl 	=> sprite_address,
    write_out_from_the_sprite_ctrl 		=> sprite_write,
    writedata_out_from_the_sprite_ctrl => sprite_writedata,
    irq_to_the_sprite_ctrl 				=> sprite_irq,

	 mousedata_to_the_mouse_ctrl 			=> mouse_data, 
	 
	 outp_from_the_mux_ctrl 				=> read_ready
    );
	 
	 
mouse: entity work.de2_ps2 port map(
    clkin			=> CLOCK_50,
	 rst        	=> not reset_n,
    ps2_clk			=> PS2_CLK, 
	 ps2_data 		=> PS2_DAT,
    lefbut			=> mouse_data(0),
	 rigbut			=> mouse_data(1),
	 midbut			=> mouse_data(2),
    x_mov			=> mouse_data(12 downto 3),
	 y_mov			=> mouse_data(22 downto 13),
	 mouse_irq		=> mouse_data(23)
	);
	
	
mux_hw_sw: entity work.mux_hw_sw port map(
    clk        	=> CLOCK_50,
    reset_n    	=> '1',
    inp		   	=> read_ready,
    chipselect 	=> SRAM_chipselect_io,
    read 			=> SRAM_read_io,
	 write 			=> SRAM_write_io,
	 
    address  		=> SRAM_address_io,
    writedata 		=> SRAM_writedata_io,
    byteenable 	=> SRAM_byteenable_io,

    readdata     	=> ram_data,
	 
    write_inp		=> '0',
	 read_inp		=> '1',
	 writedata_inp	=> (others => '0'),
	 address_inp   => ram_addr,
	 
    SRAM_DQ 		=> SRAM_DQ,
    SRAM_ADDR 		=> SRAM_ADDR,
    SRAM_UB_N		=> SRAM_UB_N,
	 SRAM_LB_N 		=> SRAM_LB_N,
    SRAM_WE_N		=> SRAM_WE_N,
	 SRAM_CE_N 		=> SRAM_CE_N,
    SRAM_OE_N 		=> SRAM_OE_N,
	 outp 			=> LEDR(17)
    );
	 
	 
sprite : entity work.sprite port map (
    clk_50       	=> CLOCK_50,    
	 address      	=> sprite_address,
    write        	=> sprite_write,
    writedata    	=> unsigned(sprite_writedata),
	 irq          	=> sprite_irq,
	 
	 pixel        	=> sprite_pixel,
	 x			 		=> sprite_x,
	 y			 		=> sprite_y,
	 y_blank		 	=> sprite_y_blank,
	 x_count      	=> x_cnt,
		
	 sram_addr    	=> ram_addr,
	 sram_datain  	=> ram_data
);
  


vga_display: entity work.de2_vga_raster port map(
    reset 		=> '0',
    clk_50 		=> CLOCK_50,
	 
	 pixel 		=> sprite_pixel, 
	 x_index 	=> sprite_x,
	 y_index 	=> sprite_y,
	 y_blank 	=> sprite_y_blank,
	 x_count 	=> x_cnt,
    inp		   => read_ready,
	 
    VGA_CLK 	=> VGA_CLK,
    VGA_HS 		=> VGA_HS,
    VGA_VS 		=> VGA_VS,
    VGA_BLANK 	=> VGA_BLANK,
    VGA_SYNC 	=> VGA_SYNC,
    VGA_R 		=> VGA_R,
    VGA_G 		=> VGA_G,
    VGA_B 		=> VGA_B
    );	 
	 
neg_3ns: entity work.sdram_pll port map(
	inclk0 => CLOCK_50, 
	c0     => DRAM_CLK, 
	c1     => pll_c1);  

  HEX7     <= "0001001"; -- Leftmost
  HEX6     <= "0000110";
  HEX5     <= "1000111";
  HEX4     <= "1000111";
  HEX3     <= "1000000";
  HEX2     <= (others => '1');
  HEX1     <= (others => '1');
  HEX0     <= (others => '1');          -- Rightmost
  --LEDG     <= (others => '1');
  --LEDR     <= (others => '1');
  LCD_ON   <= '1';
  LCD_BLON <= '1';
  LCD_RW <= '1';
  LCD_EN <= '0';
  LCD_RS <= '0';

  SD_DAT3 <= '1';  
  SD_CMD <= '1';
  SD_CLK <= '1';

  --SRAM_DQ <= (others => 'Z');
  --SRAM_ADDR <= (others => '0');
  --SRAM_UB_N <= '1';
  --SRAM_LB_N <= '1';
  --SRAM_CE_N <= '1';
  --SRAM_WE_N <= '1';
  --SRAM_OE_N <= '1';

  UART_TXD <= '0';
  --DRAM_ADDR <= (others => '0');
  DRAM_LDQM <= DQM(0);
  DRAM_UDQM <= DQM(1);
  --DRAM_WE_N <= '1';
  --DRAM_CAS_N <= '1';
  --DRAM_RAS_N <= '1';
  --DRAM_CS_N <= '1';
  DRAM_BA_0 <= BA(0);
  DRAM_BA_1 <= BA(1);
  --DRAM_CLK <= '0';
  --DRAM_CKE <= '0';

  FL_ADDR <= (others => '0');
  FL_WE_N <= '1';
  FL_RST_N <= '0';
  FL_OE_N <= '1';
  FL_CE_N <= '1';
  OTG_ADDR <= (others => '0');
  OTG_CS_N <= '1';
  OTG_RD_N <= '1';
  OTG_RD_N <= '1';
  OTG_WR_N <= '1';
  OTG_RST_N <= '1';
  OTG_FSPEED <= '1';
  OTG_LSPEED <= '1';
  OTG_DACK0_N <= '1';
  OTG_DACK1_N <= '1';

  TDO <= '0';

  ENET_CMD <= '0';
  ENET_CS_N <= '1';
  ENET_WR_N <= '1';
  ENET_RD_N <= '1';
  ENET_RST_N <= '1';
  ENET_CLK <= '0';
  
  TD_RESET <= '0';
  
  --I2C_SCLK <= '1';

  --AUD_DACDAT <= '1';
  --AUD_XCK <= '1';
  
  -- Set all bidirectional ports to tri-state
  --DRAM_DQ     <= (others => 'Z');
  FL_DQ       <= (others => 'Z');
  --SRAM_DQ     <= (others => 'Z');
  OTG_DATA    <= (others => 'Z');
  LCD_DATA    <= (others => 'Z');
  SD_DAT      <= 'Z';
  I2C_SDAT    <= 'Z';
  ENET_DATA   <= (others => 'Z');
  AUD_ADCLRCK <= 'Z';
  AUD_DACLRCK <= 'Z';
  AUD_BCLK    <= 'Z';
  GPIO_0      <= (others => 'Z');
  GPIO_1      <= (others => 'Z');

end datapath;
